// Written by Dr. YIP Chi Lap [Beta]
//   for Workshop on "Computer-aided visibility measurement using digital cameras",
//   Visibility Measurement Method Design Competition,
//   Faculty of Engineering, The University of Hong Kong, 2012.

/*
   Answer to Exercise:
     Print out the average brightness of the image
       in the rectangular area marked by the user,
       when the mouse button is released.

     Hint 1. Define a variable named "sum" that is a floating point number this way:
               float sum  = 0.0;
     Hint 2. Loop over the pixels in the area to find the brightness values
               and accumulate them to the "sum" variable,
               and divide the sum by the area.
             You need a loop within another loop (nested loops) to do that.

     => See below.
        Note that this program requires startX to be no greater than endX,
          startY to be no greater than endY, and
          all four corners of the rectangle to be within the image.
        Those who are interested to make it foolproof can make use of functions
          min(), max(), constrain(), etc.
*/


// For loading image
PImage img;

// For rectangle definition
int startX = -1;
int startY = -1;
int endX   = -1;
int endY   = -1;


void setup() {
  // Load the image
  img = loadImage("imgHKW_100605_0900.jpg");

  // Make window size same as image size.
  size(img.width, img.height);
}


void draw() {
  // Use image img as background
  background(img);

  // Draw the rectangle
  rectMode(CORNERS);
  noFill();
  stroke(0);
  rect(startX, startY, endX, endY);
}


void mouseMoved() {
  // Prepare for accessing colour of a pixel
  loadPixels();

  // Get the colour of the pixel of the picture at (mouseX,mouseY)
  color p = pixels[mouseY*img.width + mouseX];

  /* Commented out to reduce the amount of output
  // Print out the component and brightness values
  println("red: " + red(p) +
          ", green: " + green(p) +
          ", blue: " + blue(p) +
	  ", brightness: " + brightness(p));
  */
}


void mousePressed() {
  startX = mouseX;
  startY = mouseY;
  endX = mouseX;
  endY = mouseY;
}


void mouseDragged() {
  endX = mouseX;
  endY = mouseY;
}


void mouseReleased() {
  endX = mouseX;
  endY = mouseY;

  /*
     Setting the background again effectively clears the rectangle
       that has been drawn on the screen.
     Without doing that, the average brightness calculated
       would include pixels of the rectangle drawn.
  */
  background(img);

  /*
     Don't forget to do this before we access pixels[]
     Calling it once is enough, so do not put loadPixels() in a loop.
  */
  loadPixels();

  /* For temporarily accumulating brightness values */
  float sum = 0.0;

  /* 
     For each y value between startY (included) and endY (included because of the <=)
     y++ means y+=1, increments y by one
  */
  for (int y=startY; y<=endY; ++y) {
    /* for each x value between startX included and endX included */
    for (int x=startX; x<=endX; ++x) {
      color p = pixels[y*img.width+x];
      sum += brightness(p);
    }
  }
  /*
     Here, sum is the accumulated sum of the brightness values
     We now calculate the average brightness value by
       dividing the accumulated sum of the brightness values
       by the area.
     Notice that area is an integer while average brightness is a floating point number.
     The +1 in area calculation is needed because both start and end points are included.
  */
  int area = (endX-startX+1)*(endY-startY+1);
  float averageBrightness = sum / area;

  /* Print out the area and average brightness */
  println("Area = " + area);
  println("Average brightness = " + averageBrightness);
}

/*
   Challenge:
     Modify the program so that it can
       report the average brightness of the top and bottom halves
       of the rectangle drawn by the user.
*/
